package ms;

/*
 * This program is used to modify the format of default Lexicon synonyms:
 * 1. remove the no-punctuation, lowercase field (field 1)
 * 2. remove duplicated synonyms
 * 3. add missed symmetric synonyms
 * 4. Add field 1 with no-punctuation, lowercase of Field 2
 */

import java.io.*;
import java.util.*;

public class ModifyLexiconSynonyms
{
    public static void main(String[] args)
    {
        try
        {
            // Parse the inputs, default is infl.data
            String inFile = "/export/home/lu/Development/LVG/Components/SynonymPermutationTool/data/Ms/synonyms.data";
            String outFile = "/export/home/lu/Development/LVG/Components/SynonymPermutationTool/data/Ms/synonyms.out";

            if(args.length == 2) 
			{
                inFile = args[0];
                outFile = args[1];
			}
            else if(args.length != 0)
            {
          		System.err.println("Usage: java ModifyLexiconSynonyms <inFileName> <outFileName>");
				System.exit(0);
            }

            // read in data from file
            ReadSynonyms(inFile);
            PrintSynonymsForDb(outFile);
        }
        catch (Exception e)
        {
            System.err.println(e);
        }
	}

    // private methods
	// read in synonyms, modify, save, then print out
	private static void ReadSynonyms(String inFile)
    {
        String line = null;
        int lineNo = 0;
        int synonymNo = 0;

        try
        {
            BufferedReader fIn = new BufferedReader(new InputStreamReader(
				new FileInputStream(inFile), "UTF-8"));

            // read in line by line from a file
            while((line = fIn.readLine()) != null)
            {
                // skip the line if it is empty or comments (#)
                if((line.length() > 0) && (line.charAt(0) != '#'))
                {
                    // use '|' and as delimiter to parse token
                    StringTokenizer buf = new StringTokenizer(line, "|");

                    // read in data & get citations & inflections
                    String normSynonym = buf.nextToken();
                    String inSynonym = buf.nextToken();
					String inCat = buf.nextToken();
                    String outSynonym = buf.nextToken();
					String outCat = buf.nextToken();

					TermCat inTermCat = new TermCat(inSynonym, inCat); 
					TermCat outTermCat = new TermCat(outSynonym, outCat); 

					AddToSynonymList(inTermCat, outTermCat);

					synonymNo++;
				}

				// update lineNo & print out current lineNo
                lineNo++;
                if((lineNo%1000) == 0)
                {
                    System.out.println("-- Cur : " + lineNo + ", Synonyms: "
						+ synonymNo);
                }
            }
            fIn.close();

            System.out.println("-- Read from file: [" + inFile + "]: " 
				+ lineNo + " lines and found " + synonymNo + " synonyms.");
        }
        catch(Exception e)
        {
            System.err.println("Exception: " + e.toString());
            System.err.println("Line(" + lineNo + "):" + line);
        }
    }

	private static void AddToSynonymList(TermCat inSynonym, TermCat outSynonym)
	{
		// add to synonym list if inSynonym does not exist
		if(IsSynonymExist(inSynonym, outSynonym) == false)
		{
			// add to synonym list
            Synonym synonym1 = new Synonym(inSynonym, outSynonym);
            Synonym synonym2 = new Synonym(outSynonym, inSynonym);
            synonymList_.addElement(synonym1);
            synonymList_.addElement(synonym2);

            // update synonym index
            int listSize = synonymList_.size();
            int index1 = listSize - 2;
            int index2 = listSize - 1;
            AddSynonymIndex(inSynonym, index1);
            AddSynonymIndex(outSynonym, index2);
		}
	}

	private static void AddSynonymIndex(TermCat synonym, int index)
    {
        Integer indexObj = new Integer(index);
        Vector<Integer> indexList = synonymIndex_.get(synonym);
        if(indexList == null)   // add new index to index list
        {
            Vector<Integer> newIndexList = new Vector<Integer>();
            newIndexList.addElement(indexObj);
            synonymIndex_.put(synonym, newIndexList);
        }
        else    // add index to exist index list
        {
            synonymIndex_.get(synonym).addElement(indexObj);
        }
    }

	private static boolean IsSynonymExist(TermCat inSynonym, 
		TermCat outSynonym)
	{
        boolean existFlag = false;

        Vector<Integer> indexList = synonymIndex_.get(inSynonym);
        if(indexList != null)   // same in synonym
        {
            // go through each indexed out synonym
            for(int i = 0; i < indexList.size(); i++)
            {
                int curIndex = indexList.elementAt(i).intValue();
                TermCat curOutSynonym
                    = synonymList_.elementAt(curIndex).GetOutSynonym();
                if(outSynonym.equals(curOutSynonym) == true)
                {
                    existFlag = true;
                    break;
                }
            }
        }

		return existFlag;
    }

	private static void PrintSynonyms(String outFile)
	{
        try
        {
			BufferedWriter fOut = new BufferedWriter(new OutputStreamWriter(
			    new FileOutputStream(outFile), "UTF-8"));

			// print out
			for(int i = 0; i < synonymList_.size(); i++)
			{
				Synonym tempSynonym = synonymList_.elementAt(i);

				fOut.write(tempSynonym.ToString());
				fOut.newLine();
			}

            System.out.println("-- Print out to file: [" + outFile
				+ "] " + synonymList_.size() + " synonyms.");

			fOut.close();
        }
        catch(Exception e)
        {
            System.err.println("Exception: " + e.toString());
        }
	}

	// add 1st field with no-punctuation, lowercase of Field 2
	private static void PrintSynonymsForDb(String outFile)
	{
        try
        {
			BufferedWriter fOut = new BufferedWriter(new OutputStreamWriter(
			    new FileOutputStream(outFile), "UTF-8"));

			// print out
			for(int i = 0; i < synonymList_.size(); i++)
			{
				Synonym tempSynonym = synonymList_.elementAt(i);

				fOut.write(tempSynonym.ToDbString());
				fOut.newLine();
			}

            System.out.println("-- Print out to file: [" + outFile
				+ "] " + synonymList_.size() + " synonyms.");

			fOut.close();
        }
        catch(Exception e)
        {
            System.err.println("Exception: " + e.toString());
        }
	}

    // data member
	final private static String SP = "|";

	private static Vector<Synonym> synonymList_ = new Vector<Synonym>();
	private static Hashtable<TermCat, Vector<Integer>> synonymIndex_
		= new Hashtable<TermCat, Vector<Integer>>();

}
